#include <iostream>

#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/mman.h>

#include <stdlib.h>

#include "hash.h"
#include "seq_pair.h"
#include "version.h"

using namespace std;

#ifndef PARA_H
#define PARA_H

bool _in_pair = true; 

bool _single_input = true; 

bool _keep_pair = true;

bool _dup = true;

bool _ext_qual = false; //if indenpent qual

int _trim_begin = 0;
int _trim_end = 0;

int _trim_qual_orig = 0;
int _trim_qual = 0;
float _trim_p = 0.8; 
bool _trim_qual_format = true; //true: Sanger; false: Phred;

float _gc_p_min = 0;
float _gc_p_max = 1;

string infilename_1, infilename_2;
string extqualname_1, extqualname_2; 

string inprimer;
string outfilename = "Default_QC";

bool _in_format = true; //true : fastq; false : fasta;

bool _out_format = true; //true : fastq; false : fasta;

int _thread = 1;

vector <string> order;

hash_map <string, Seq_pair, std_string_hash> seq_pairs;

string primers[MAX_PRIMER];
int primer_count = 0;

void Print_help(){
     
     cout << Name << endl;
     cout << "Version : " << Version << endl;
     
     cout << "Usage :" << endl;
     cout << "\t parallel-qc [options] value" << endl;
     cout << "Options:" << endl;
     cout << "\t-i Input file name(s) [Required]" << endl;
     cout << "\t  Input file must be in FASTQ or FASTA format, supporting 1 or 2 names." << endl;
     cout << endl;
     
     cout << "\t-Q quality file name(s)" << endl; 
     cout << "\t  Available only when the input is in FASTA format, supporting 1 or 2 names." << endl;
     
     cout << "\t-o Output file directory [Required]" << endl;
     
     cout << "\t-f Q or F " << endl;
     cout << "\t  Output format. Q for FASTQ and F for FASTA. Default is FASTQ" << endl;
     
     cout << "\t-p T or F" << endl;
     cout << "\t  If the input is pair-end reads. Default is T." << endl;
     
     cout << "\t-k T or F" << endl;
     cout << "\t  If keep the pairs for output. Default is T." << endl;
     
     cout << "\t-d T or F" << endl;
     cout << "\t  If drop the duplicated reads. Default is T" << endl;
     
     cout << "\t-b begin(int) end(int)" << endl;
     cout << "\t  Base trim begin and end. Keep 0 0 to turn off. Default is turn off." << endl;
     cout << endl;
     
     cout << "\t-q quality(int) ratio(float, 0~1), default in Sanger format. Add '-P' if in Illumina 1.3 format." << endl;
     cout << "\t  Trim quality value and threshold ratio that minimum percentage of bases must have the trim quality value. Keep 0 0 to turn off. Default is turn off" << endl;
     cout << endl;
     
     cout << "\t-m Tag sequences file, must be in FASTA format. Maximun sequence number is 100." << endl;
     
     cout << "\t-g GC min_ratio(float, 0~1) max_ratio(flaot, 0~1)." << endl;
     cout << "\t  GC proportion trim, must between min_ratio ~ max_ratio. Default is turn off" << endl; 
     cout << endl;
     
     cout << "\t-t thread number. Default thread number is 1." << endl;
     
     cout << "\t-h Print help." << endl;    
     
     exit(0);
     
     }

void Para_args(int argc, char * argv[]){
    
    int i = 1;
      
    if (argc ==1) 
		Print_help();
      
    while(i < argc){
         if (argv[i][0] != '-') {
                           printf("Argument # %d Error : Arguments must start with -\n", i);
                           exit(0);
                           };           
         switch(argv[i][1]){
                            case 'i': 
                                      infilename_1 = argv[i+1]; 
                                      if (argv[i+2][0] != '-'){ 
                                                                infilename_2 = argv[++i +1];
                                                                _single_input = false;
                                                                }
                                      break; //input
                            
                            //case 'F': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _in_format = false; break; //out_format
                                 
                            case 'Q': extqualname_1 = argv[i+1]; 
                                      _ext_qual = true;
                                      if (argv[i+2][0] != '-') 
                                                                extqualname_2 = argv[++i +1];                                                                
                                                                
                                      break; //extern qual
                                      
                            case 'o': outfilename = argv[i+1]; break; //output
                            
                            case 'p': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _in_pair = false; _keep_pair = false; _single_input = true; break; //in pair                            
                            case 'k': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _keep_pair = false; break; //keep pair
                            case 'd': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _dup = false; break; //dup
                            case 'b': _trim_begin = atoi(argv[i+1]) - 1; _trim_end = atoi(argv[++i +1]) - 1; break; //trim
                            
                            case 'q': _trim_qual_orig = atoi(argv[i+1]); 
                                      _trim_p = atof(argv[++i +1]);
                                      if ((i + 2)>= argc) break;  
                                      if (argv[i+2][1] == 'P'){ 
                                         _trim_qual = _trim_qual_orig + 64;
                                         _trim_qual_format = false;
                                         i ++;
                                         }  // Illunima 1.3 + 64
                                      else 
                                           _trim_qual = _trim_qual_orig + 33; //Default Sanger + 33   
                                      break; //qual
                            
                            case 'm': inprimer = argv[i+1]; break;
                            
                            case 'g': _gc_p_min = atof(argv[i+1]); _gc_p_max = atof(argv[++i +1]); break;
                            
                            case 'f': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _out_format = false; break; //out_format
                            case 't': _thread = atoi(argv[i+1]); break;
                            
                            case 'h': Print_help(); break;

                            default : printf("Unrec argument %s\n", argv[i]); Print_help(); break; 
                            }
         i+=2;
         }
    
    _in_format = Check_format(infilename_1.c_str()); 
    
    Check_Path(outfilename, 1);
    
    if (_in_format && _ext_qual) {
                   
                   cerr << "Waring : Fastq format input with extern quality which will be ignored" << endl;
                   
                   _ext_qual = false;
                   
                   }
    
    if (!_in_format && !_ext_qual){
                    
                    cerr << "Waring : Fasta format input without extern quality" << endl;
                    _out_format = false;
                    }
    
    }

#endif
