#include <iostream>
#include <fstream>
#include <map>
#include <vector>

#include <stdlib.h>

#include "hash.h"
#include "version.h"
#include "seq_pair.h"

#ifndef RANDOM_H
#define RANDOM_H

using namespace std;

string infilename_1, infilename_2;
string extqualname_1, extqualname_2;

string outfilename = "Default_Ran_Sel";

bool _in_pair = true;
bool _single_input = true;
bool _in_format = true;
bool _out_format = true;
bool _ext_qual = false;

int reads_count = 0;

float select_p = 0.0;
int select_count = 0;

vector <string> order;

hash_map <string, Seq_pair, std_string_hash> seq_pairs;

int * select_order;

void Print_help(){
     
     cout << Name << endl;
     cout << "Rand-sel" << endl;
     cout << "Version : " << Version << endl;
     
     cout << "Usage :" << endl;
     cout << "\t rand-sel [options] value" << endl;
     cout << "Options:" << endl;
     cout << "\t-i Input file name(s)[Required]" << endl;
     cout << "\t  Input file must be in FASTQ or FASTA format, supporting 1 or 2 names." << endl;
     
     cout << "\t-Q quality file name(s)" << endl; 
     cout << "\t  Available only when the input is in FASTA format, supporting 1 or 2 names." << endl;
     
     cout << "\t-o Output file name prefix [Required]" << endl;
     
     cout << "\t-f Q or F " << endl;
     cout << "\t  Output format. Q for FASTQ and F for FASTA. Default is FASTQ" << endl;
     
     cout << "\t-p T or F" << endl;
     cout << "\t  If the input is pair-end reads. Default is T." << endl;
     
     cout << "\t-P select_ratio(float, 0-1)" << endl;
     cout << "\t  Random selection proportion." << endl; 
     
     cout << "\t-h Print help." << endl;    
     
     exit(0);
     
     }

void Para_args(int argc, char * argv[]){
     
     int i = 1;
      
    if (argc ==1) 
		Print_help();
      
    while(i < argc){
         if (argv[i][0] != '-') {
                           printf("Argument # %d Error : Arguments must start with -\n", i);
                           exit(0);
                           };           
         switch(argv[i][1]){
                            case 'i': 
                                      infilename_1 = argv[i+1]; 
                                      if (argv[i+2][0] != '-'){ 
                                                                infilename_2 = argv[++i +1];
                                                                _single_input = false;
                                                                }
                                      break; //input
                            
                            //case 'F': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _in_format = false; break; //out_format
                                 
                            case 'Q': extqualname_1 = argv[i+1]; 
                                      if (argv[i+2][0] != '-'){ 
                                                                extqualname_2 = argv[++i +1];
                                                                _ext_qual = true;
                                                                }
                                      break; //extern qual
                                      
                            case 'o': outfilename = argv[i+1]; break; //output
                            
                            case 'p': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _in_pair = false; _single_input = true; break;//in pair                                                                                                                                            
                            
                            case 'f': if ((argv[i+1][0] == 'f') || (argv[i+1][0] == 'F')) _out_format = false; break; //out_format
                            
                            case 'P': select_p = atof(argv[i+1]); break;
                            
                            case 'h': Print_help(); break;

                            default : printf("Unrec argument %s\n", argv[i]); Print_help(); break; 
                            }
         i+=2;
         }
    
    _in_format = Check_format(infilename_1.c_str()); 
    
    if (_in_format && _ext_qual) {
                   
                   cerr << "Waring : Fastq format input with extern quality which will be ignored" << endl;
                   
                   _ext_qual = false;
                   
                   }
    
    if (!_in_format && !_ext_qual){
                    
                    cerr << "Waring : Fasta format input without extern quality" << endl;
                    _out_format = false;
                    }
     
     
     }

int Get_Random(int total, int n, int * order){
    
    srand((int)time(NULL));
    
    int * table = new int [total];
    int * order_table = new int [total];
    
    for(int i = 0; i< total; i++){
            table[i] = i;
            order_table[i] = 0;
            }
    for (int i = 0; i < n; i++ ){
         
         int j=(int)((float) (total-1-i)* rand()/(RAND_MAX+1.0)); 
         order_table[table[j]] = 1;
         table[j] = table[total-1-i];
         order[i] = 0;
        }
    
    int j = 0;
    
    for (int i = 0; i< total; i++)
        if (order_table[i] == 1){
           order[j] = i;
           j++;
        }
        
    return j;
}

void Output_reads_pair(){
     
     string out_surffix = ".fastq";
    
     if (!_out_format) out_surffix = ".fasta";
     
     string outfilename_1 = outfilename + "-1" + out_surffix;
     string outfilename_2 = outfilename + "-2" + out_surffix;
     
     ofstream outfile_1(outfilename_1.c_str(), ofstream::out);
     
     if (!outfile_1){
                     
                     cerr << "Error : Open output file error : " << outfilename_1 << endl;
                     exit(0);
                     
                     }
     
     ofstream outfile_2(outfilename_2.c_str(), ofstream::out);
    if (!outfile_2){
                   
                   cerr << "Error : Open output file error : " << outfilename_2 << endl;
                   exit(0);
                   
                   }
    
    int count_order = 0;
    int count_select_order = 0;
    
    for (vector <string>::iterator viter = order.begin(); viter != order.end(); viter ++){
        
        if (count_order == select_order[count_select_order]){
                        
                        string label = *viter;                        
                        
                        if (seq_pairs.count(label)== 0){
                                             
                                             cerr << "Waring : ID " << label << " Cannot be found" << endl;
                                             
                                             }
                          
                        Seq_pair seq_pair = seq_pairs[label];   
                        
                        seq_pair.Out_put(label, & outfile_1, & outfile_2, _out_format);
                        
                        count_select_order ++;
                        
                        } 
        
        if (count_select_order == select_count) break;
        
        count_order ++;
        }
    
    //cout << count_select_order << endl;
    
    return;
}
    
void Output_reads(){
     
     string out_surffix = ".fastq";
    
     if (!_out_format) out_surffix = ".fasta";
     
     string outfilename_1 = outfilename + "" + out_surffix;
     
     ofstream outfile_1(outfilename_1.c_str(), ofstream::out);
     
     if (!outfile_1){
                     
                     cerr << "Error : Open output file error : " << outfilename_1 << endl;
                     exit(0);
                     
                     }          
    
    int count_order = 0;
    int count_select_order = 0;
    
    for (vector <string>::iterator viter = order.begin(); viter != order.end(); viter ++){
        
        if (count_order == select_order[count_select_order]){
                        
                        string label = *viter;
                  
                        if (seq_pairs.count(label)== 0){
                                             
                                             cerr << "Waring : ID " << label << " Cannot be found" << endl;
                                             
                                             }
                          
                        Seq_pair seq_pair = seq_pairs[label];   
                        
                        seq_pair.Out_put(label, & outfile_1, NULL, _out_format);
                        
                        count_select_order ++;
                        
                        } 
        
        if (count_select_order == select_count) break;
        
        count_order ++;
        }
    
    return;
}
     
#endif    
